/*
    COS.C
*/

#define _LIBM_SOURCE

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

double cos(x)
double x;

/* Uses series from Abramowitz & Stegun 4.3.66 as direct use of series
* gives greater accuracy than using sin( x - pi/2 )
* S E Peterson 3/92 */

{
    static double coscoef[11] = {
                         1.0,
                        -0.5,
                         4.16666666666666666667E-2,
                        -1.38888888888888888889E-3,
                         2.48015873015873015873E-5,
                        -2.75573192239858906526E-7,
                         2.08767569878680989792E-9,
                        -1.14707455977297247139E-11,
                         4.77947733238738529744E-14,
                        -1.56192069685862264622E-16,
                         4.11031762331216485848E-18
                         }; 
    double x2, ans;
    int i;
    double theta, sign;

    sign = 1.0;
    theta = fmod(x, PI2);
    if (theta < 0.0)
        theta = -theta;
    if (theta > PI){ 
        theta  -= PI;
        sign = -1.0;
    } 

    x2 = theta * theta;
    ans = coscoef[10];
    for (i = 9; i >= 0; i--)
        ans = x2 * ans + coscoef[i];

    return ((ans * sign));
}   

