/*
 *      _ L I S T _ F i f o R e m o v e
 *
 *  This is used to remove a node from a 
 *  singled linked list that does NOT have
 *  a pointer to the last node (FIFO).
 *
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  23 May 96   DJW   - First version
 *
 *  28 May 96   DJW   - Updated 'remove' to allow for the user providing
 *                      a member to remove who is not on the list!
 */


#include <liblist.h>
#include "liblistx.h"


int     _LIST_FifoRemove (list_t list, node_t node)
{
	_fifoh_t fifolist = (_fifoh_t)list;
	_fifo_t fifonode = (_fifo_t)node;
    _fifo_t listnode = NULL;

	if (fifonode == NULL)
	{
		return LIST_ERROR_BADNODE;
	}
    /*
     *  If this is the first node, then
     *  we have an easy life!
     */
    if (fifonode == fifolist->nodes)
    {
        fifolist->nodes = fifonode->next;
    }
    else
    {
        /*
         *  We need to scan the whole list in sequence
         */
        for (listnode = fifolist->nodes ; listnode != NULL ; listnode = listnode->next)
        {
            if (listnode->next == fifonode)
            {
                break;
            }
        }
        /*
         *  We had better check that we found it!
         *  The user could give us a member not attached
         *  (or a NULL value which has the same effect).
         */
        if (listnode == NULL)
        {
			return LIST_ERROR_NOTFOUND;
        }
        listnode->next = fifonode->next;
    }
    /*
	 * If this was the last node, then
	 * we had better set that as well.
	 */
    if (fifolist->last == fifonode)
    {
        LIBASSERT (fifonode->next == NULL)
        fifolist->last = listnode;
    }
    /*
     *  Now do the tidying up.
     */
    fifonode->next = NULL;
    return (LIST_ERROR_NONE);
}

