/*
 *          _ L I S T _ L i f o A f t e r
 *
 *  This routine adds a node on a list that
 *  DOES NOT maintain a pointer to the last node
 *
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  23 May 96   DJW   - First version
 */


#include <liblist.h>
#include "liblistx.h"


int     _LIST_LifoAfter (list_t list, node_t newnode, node_t oldnode)
{
	register _lifoh_t lifolist = (_lifoh_t)list;
	register _lifo_t  lifonewnode = (_lifo_t)newnode;
	register _lifo_t  lifooldnode = (_lifo_t)oldnode;

    LIBASSERT (newnode != NULL)
    LIBASSERT (lifonewnode->next == NULL)

    /*
     * Empty list ?
     */
    if (lifolist->nodes == NULL)
    {
        LIBASSERT (lifooldnode == NULL)

        lifolist->nodes = lifonewnode;
    }
    else
    {
        if (lifooldnode == NULL)
        {
            /*
             *  At start of existing list
             */
            lifonewnode->next = lifolist->nodes;
            lifolist->nodes = lifonewnode;
        }
        else
        {
            /*
             *  Into middle of existing list
             */

            lifonewnode->next = lifooldnode->next;
            lifooldnode->next = lifonewnode;
        }
    }
    return (LIST_ERROR_NONE);
}

