/*
 *      L I S T _ N e w A f t e r
 *
 *  This function is called when it is required
 *  to create a new node and immediately insert it
 *  after a specifed node.
 *
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 */


#include <liblist.h>
#include "liblistx.h"

/*
 *  This function combines the LIST_New() functionality
 *  with that of the LIST_After() method in one call.
 */
node_t    LIST_NewAfter (list_t list, node_t oldnode, ...)
{
	_listmaster_t listmaster;
	listclass_t   listclass;
	node_t  newnode;
    int     reply;
    va_list ap;

    LIBDEBUG ("LIST_NewAfter", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListCheck(list))

	listclass = list->listclass;
	listmaster = listclass->master;

    if (oldnode == NULL)
    {
        oldnode = listmaster->last(list);
    }
#ifdef DEBUG
    if (oldnode != NULL)
    {
        LIBDEBUG ("LIST_NewAfter", \
               "'oldnode' is not a valid member of this list", \
               _LIST_NodeCheck(list,oldnode))
        LIBDEBUG  ("LIST_NewAfter", \
               "'oldnode' does not exist in this list", \
               LIST_Index(list,oldnode))
    }
#endif /* DEBUG */


    /*
     *  Check that a node initialisation
     *  routine has been provided.
     */
    if (listclass->initnode == NULL)
    {
        errno = LIST_ERROR_BADINIT;
        return NULL;
    }
    /*
     *  Create a new node
     *  (passing on variable parameter list)
     */
    va_start (ap, oldnode);
    newnode = listmaster->nodenew(list, ap);
    va_end(ap);
    if (newnode == NULL)
    {
        return (NULL);
    }
    reply = LIST_After(list,newnode,oldnode);
    if (reply != LIST_ERROR_NONE)
    {
        errno = reply;
#ifdef LIBTEST
        LIBASSERT (LIST_FreeNode(list,&newnode) == LIST_ERROR_NONE)
#else
        (void)LIST_FreeNode(list,&newnode);
#endif /* LIBTEST */
        return (NULL);
    }
    return (newnode);
}

