/*
 *      L I S T _ N e w P u s h
 *
 *  This function is called when it is required
 *  to create a new node and insert it at the 
 *  beginning of a list being treated as a stack.
 *
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 */


#include <liblist.h>
#include "liblistx.h"


/*
 *  This function combines the LIST_New() functionality
 *  with that of the LIST_Push() method in one call.
 */
node_t    LIST_NewPush (list_t list, ...)
{
    listclass_t listclass;
    node_t  newnode;
    int     reply;
    va_list ap;

    LIBDEBUG ("LIST_NewPush", \
            "'list' parameter is not a valid list pointer", \
            _LIST_ListCheck(list))


	listclass = list->listclass;

	/*
     *  Check that a node initialisation
     *  routine has been provided.
     */
    if (listclass->initnode == NULL)
    {
        errno = LIST_ERROR_BADINIT;
        return NULL;
    }
    /*
     *  Create a new node
     *  (passing on variable parameter list)
     */
    va_start (ap, list);
    newnode = listclass->master->nodenew(list, ap);
    va_end(ap);
    if (newnode == NULL)
    {
        return (NULL);
    }
    reply = LIST_Push(list,newnode);
    if (reply != LIST_ERROR_NONE)
    {
        errno = reply;
#ifdef LIBTEST
        LIBASSERT (LIST_FreeNode(list,&newnode) == LIST_ERROR_NONE)
#else
        (void)LIST_FreeNode(list,&newnode);
#endif
        return (NULL);
    }
    return (newnode);
}


